__extensions [ "sound.jar" ]

globals 
[
  clock tick-length          ;; clock variables
  box-edge                   ;; distance of box edge from axes
  pressure
  pressure-history
  zero-pressure-count        ;; how many zero entries are in pressure-history
  wall-hits-per-particle     ;; average number of wall hits per particle
  particles-to-add
  new-particles              ;; agentset of particles added via add-particles-middle
  message-shown?             ;; whether we've shown the warning message yet
  length-horizontal-surface  ;; the size of the wall surfaces that run horizontally - the top and bottom of the box
  length-vertical-surface    ;; the size of the wall surfaces that run vertically - the left and right of the box
  three-speed                ;; current speed of particle 3 (music addition)
  pressure-now               ;; current pressure (music addition
]

breeds [ particles flashes clocker ]

flashes-own [birthday]

particles-own 
[
  speed mass                 ;; particle info
  wall-hits                  ;; # of wall hits during this clock cycle ("big tick")
  momentum-difference        ;; used to calculate pressure from wall hits
  last-collision
  new?                       ;; used to build the new-particles agentset; this is
                             ;; only ever set to true by add-particles-middle
]

to startup
  set message-shown? false
end

to setup
  locals [tmp]
  stop-music  ;; start music by closing previous one
  set tmp message-shown?
  ca
  set message-shown? tmp
  set-default-shape particles "circle"
  set clock 0
  set particles-to-add 0
  ;; box has constant size...
  set box-edge (screen-edge-x - 1)
  ;;; the length of the horizontal or vertical surface of 
  ;;; the inside of the box must exclude the two patches 
  ;; that are the where the perpendicular walls join it, 
  ;;; but must also add in the axes as an additional patch
  ;;; example:  a box with a box-edge of 10, is drawn with 
  ;;; 19 patches of wall space on the inside of the box
  set length-horizontal-surface  ( 2 * (box-edge - 1) + 1) 
  set length-vertical-surface  ( 2 * (box-edge - 1) + 1) 
  make-box
  make-particles initial-number
  make-clocker
  set pressure-history []
  set zero-pressure-count 0
  setup-plots
  do-plotting
  set pressure-now pressure ;; setup for current pressure
  set three-speed speed-of particle 3  ;; setup for current particle speed
end

to go 
  locals [old-clock ]
  if single-on? 
    [ set single-particle-speed? true set single-particle-collisions? true set single-particle-wall-hits? true ]    
  if group-on?
     [ set pressure? true set wall-hits? true set collisions? true ]
   if not single-particle-speed? and three-speed > 0  ;; making sure that particle speed is sung only when switch is on
        [ stop-note "oboe" (80 - 120 / ( three-speed ))
          stop-note "oboe" 50 ]
   if not pressure? and pressure > 0  ; making sure that pressure is sung only when switch is on
        [ stop-note "recorder" ( 100 - 2000 / pressure )
          stop-note "recorder" 20 ]
   if single-particle-speed?
     [ ask particle 3  ;; asking particle 3 to sing, changes note when speed changes, taking care of clock = 0
         [
           ifelse  ((abs (speed - three-speed) > 0 ) )
                     or (abs (speed - three-speed ) = speed )
              [ sing-one-particle-speed ]
              [ ifelse clock > 0 
                [ stop ] 
                [ start-note "oboe" (80 - 120 / ( speed-of particle 3 )) single-particle-loudness - 10]
               ]
          ]
      ]
   if real-time? [ sing-real-time ]  ;; sing the real time ticker
   ask particles
      [ set new? false ]
    ask particles [ bounce ]
    ask particles [ move ]
    ask particles
      [ ;; without-interruption is needed here so one collision is
        ;; happening at a time
        without-interruption
          [ check-for-collision ] ]
    add-particles-side 
    set old-clock clock
    set clock clock + tick-length
    if floor clock > floor (clock - tick-length)
      [ ifelse any? particles
          [ set wall-hits-per-particle mean values-from particles [wall-hits] ]
          [ set wall-hits-per-particle 0 ]
        ask particles
          [ set wall-hits 0 ]
    calculate-pressure
        do-plotting
        if model-time? [ play-note "nylon string guitar" 26 83 0.1 ] ] ;; sing model time
    calculate-tick-length
    ask clocker [ set heading clock * 360 ]
    ask flashes with [clock - birthday > 0.4]
      [ set pcolor yellow
        die ]
    ifelse (single-particle-speed? or single-particle-collisions? or single-particle-wall-hits? )  ;; single particle speed is traced
      [ ask particle 3 [ pd ] ]
      [ ask particle 3 [ pu ] ]
    fade-patches ;; if the single particle is tracing, then the trace disappears after a while
end

to calculate-tick-length
  ifelse any? particles with [speed > 0]
    [ set tick-length 1 / (ceiling max values-from particles [speed]) ]
    [ set tick-length 1 ]
end

;;; Pressure is defined as the force per unit area.  In this context,
;;; that means the total momentum per unit time transferred to the walls
;;; by particle hits, divided by the surface area of the walls.  (Here
;;; we're in a two dimensional world, so the "surface area" of the walls
;;; is just their length.)  Each wall contributes a different amount 
;;; to the total pressure in the box, based on the number of collisions, the
;;; direction of each collision, and the length of the wall.  Conservation of momentum 
;;; in hits ensures that the difference in momentum for the particles is equal to and
;;; opposite to that for the wall.  The force on each wall is the rate of change in 
;;; momentum imparted to the wall, or the sum of change in momentum for each particle: 
;;; F = SUM  [d(mv)/dt] = SUM [m(dv/dt)] = SUM [ ma ], in a direction perpendicular to 
;;; the wall surface.  The pressure (P) on a given wall is the force (F) applied to that 
;;; wall over its surface area.  The total pressure in the box is sum of each wall's
;;; pressure contribution.  

to calculate-pressure
  ;; by summing the momentum change for each particle, 
  ;; the wall's total momentum change is calculated
  set pressure 15 * sum values-from particles [momentum-difference] 
  set pressure-history lput pressure pressure-history
  set zero-pressure-count length filter [? = 0] pressure-history
  ask particles
    [ set momentum-difference 0 ]  ;; once the contribution to momentum has been calculated
                                   ;; this value is reset to zero till the next wall hit 
  ifelse pressure? ;; when pressure changes, it sings
    [ sing-pressure ]
    [ ifelse pressure-now > 0 
      [ stop-note "recorder" ( 100 - 2000 / pressure-now )
        stop-note "recorder" ( 30 ) ]
      [ stop ]
    ]
end

to bounce  ;; particle procedure
  locals [new-px new-py pitch]
  set pitch heading
  ;; if we're not about to hit a wall (yellow patch), or if we're already on a
  ;; wall, we don't need to do any further checks
  if shade-of? yellow pcolor or not shade-of? yellow pcolor-of patch-at dx dy
    [ stop ]
  ;; get the coordinates of the patch we'll be on if we go forward 1
  set new-px round (xcor + dx)
  set new-py round (ycor + dy)
  ;; if hitting left or right wall, reflect heading around x axis
  if (abs new-px = box-edge)
    [ set heading (- heading)
      set wall-hits wall-hits + 1 
      if (wall-hits?) [ play-note "celesta" pitch wall-hits-loudness 0.15 ]] ;; when a particle hits the wall there's  sound (macro)
  if (single-particle-wall-hits? and who = 3) ;; when a particle hits the wall there's  sound (micro)
        [ ask particle 3 [ pd ] play-note "clavi" ( 30 + heading / 5 ) wall-hits-loudness 0.2 ]

  ;;  if the particle is hitting a vertical wall, only the horizontal component of the speed
  ;;  vector can change.  The change in velocity for this component is 2 * the speed of the particle,
  ;; due to the reversing of direction of travel from the collision with the wall
      set momentum-difference momentum-difference + (abs (sin heading * 2 * mass * speed) / length-vertical-surface) 
  ;; if hitting top or bottom wall, reflect heading around y axis
  if (abs new-py = box-edge)
    [ set heading (180 - heading)
      set wall-hits wall-hits + 1 
      if wall-hits? [ play-note "celesta" heading  single-particle-loudness 0.1 ]
  if (single-particle-wall-hits? and who = 3)  
     [ ask particle 3 [ pd ] play-note "clavi" ( 30 + heading / 5 ) single-particle-loudness 0.1 ]
     
  ;;  if the particle is hitting a horizontal wall, only the vertical component of the speed
  ;;  vector can change.  The change in velocity for this component is 2 * the speed of the particle,
  ;; due to the reversing of direction of travel from the collision with the wall
      set momentum-difference momentum-difference + (abs (cos heading * 2 * mass * speed) / length-horizontal-surface)  ]
  ask patch new-px new-py
    [ sprout 1 [ ht
                 set breed flashes
                 set birthday clock
                 stamp yellow - 3 ] ]
end

to move  ;; particle procedure
  if patch-ahead (speed * tick-length) != patch-here
    [ set last-collision nobody ]
  jump (speed * tick-length)
end

to check-for-collision  ;; particle procedure
  locals [ candidate ]
  
  ;; Here we impose a rule that collisions only take place when there
  ;; are exactly two particles per patch.  We do this because when the
  ;; student introduces new particles from the side, we want them to
  ;; form a uniform wavefront.
  ;;
  ;; Why do we want a uniform wavefront?  Because it is actually more
  ;; realistic.  (And also because the curriculum uses the uniform
  ;; wavefront to help teach the relationship between particle collisions,
  ;; wall hits, and pressure.)
  ;;
  ;; Why is it realistic to assume a uniform wavefront?  Because in reality,
  ;; whether a collision takes place would depend on the actual headings
  ;; of the particles, not merely on their proximity.  Since the particles
  ;; in the wavefront have identical speeds and near-identical headings,
  ;; in reality they would not collide.  So even though the two-particles
  ;; rule is not itself realistic, it produces a realistic result.  Also,
  ;; unless the number of particles is extremely large, it is very rare
  ;; for three or more particles to land on the same patch (for example,
  ;; with 400 particles it happens less than 1% of the time).  So imposing
  ;; this additional rule should have only a negligible effect on the
  ;; aggregate behavior of the system.
  ;;
  ;; Why does this rule produce a uniform wavefront?  The particles all
  ;; start out on the same patch, which means that without the only-two
  ;; rule, they would all start colliding with each other immediately,
  ;; resulting in much random variation of speeds and headings.  With
  ;; the only-two rule, they are prevented from colliding with each other
  ;; until they have spread out a lot.  (And in fact, if you observe
  ;; the wavefront closely, you will see that it is not completely smooth,
  ;; because some collisions eventually do start occurring when it thins out while fanning.)

  if count other-particles-here = 1
  [
    ;; the following conditions are imposed on collision candidates:
    ;;   1. they must have a lower who number than my own, because collision
    ;;      code is asymmetrical: it must always happen from the point of view
    ;;      of just one particle.
    ;;   2. they must not be the same particle that we last collided with on
    ;;      this patch, so that we have a chance to leave the patch after we've
    ;;      collided with someone.
    set candidate random-one-of other-particles-here with
      [who < who-of myself and myself != last-collision]
    ;; we also only collide if one of us has non-zero speed. It's useless
    ;; (and incorrect, actually) for two particles with zero speed to collide.
    if (candidate != nobody) and (speed > 0 or speed-of candidate > 0)
    [
      collide-with candidate
      set last-collision candidate
      set last-collision-of candidate self
      if collisions? ;; make a sound when there's a collision that is a function of the sum of their speeds (macro)
        [ play-note "telephone ring" 2 * (speed-of self + speed-of candidate) collisions-loudness 0.15 ] 
      if (single-particle-collisions? and ( who = 3 or who-of candidate = 3 )) ;; make a sound when there's a collision that is a function of the sum of their speeds (micro)
         [ ask particle 3 [ pd ] play-note "glockenspiel" 69 single-particle-loudness + 40 0.2 ]
         
    ]
  ]
end

;; implements a collision with another particle.
;;
;; THIS IS THE HEART OF THE PARTICLE SIMULATION, AND YOU ARE STRONGLY ADVISED
;; NOT TO CHANGE IT UNLESS YOU REALLY UNDERSTAND WHAT YOU'RE DOING!
;;
;; The two particles colliding are self and other-particle, and while the
;; collision is performed from the point of view of self, both particles are
;; modified to reflect its effects. This is somewhat complicated, so I'll
;; give a general outline here: 
;;   1. Do initial setup, and determine the heading between particle centers
;;      (call it theta).
;;   2. Convert the representation of the velocity of each particle from
;;      speed/heading to a theta-based vector whose first component is the
;;      particle's speed along theta, and whose second component is the speed
;;      perpendicular to theta.
;;   3. Modify the velocity vectors to reflect the effects of the collision.
;;      This involves:
;;        a. computing the velocity of the center of mass of the whole system
;;           along direction theta
;;        b. updating the along-theta components of the two velocity vectors.
;;   4. Convert from the theta-based vector representation of velocity back to
;;      the usual speed/heading representation for each particle.
;;   5. Perform final cleanup and update derived quantities.
to collide-with [ other-particle ] ;; particle procedure
  locals
  [
    ;; local copies of other-particle's relevant quantities
    mass2 speed2 heading2 

    ;; quantities used in the collision itself
    theta   ;; heading of vector from my center to the center of other-particle.
    v1t     ;; velocity of self along direction theta
    v1l     ;; velocity of self perpendicular to theta
    v2t v2l ;; velocity of other-particle, represented in the same way
    vcm     ;; velocity of the center of mass of the colliding particles,
            ;;   along direction theta
  ]


  
  ;;; PHASE 1: initial setup

  ;; for convenience, grab some quantities from other-particle
  set mass2 mass-of other-particle
  set speed2 speed-of other-particle
  set heading2 heading-of other-particle

  ;; since particles are modeled as zero-size points, theta isn't meaningfully
  ;; defined. we can assign it randomly without affecting the model's outcome.
  set theta (random-float 360)



  ;;; PHASE 2: convert velocities to theta-based vector representation

  ;; now convert my velocity from speed/heading representation to components
  ;; along theta and perpendicular to theta
  set v1t (speed * cos (theta - heading))
  set v1l (speed * sin (theta - heading))

  ;; do the same for other-particle
  set v2t (speed2 * cos (theta - heading2))
  set v2l (speed2 * sin (theta - heading2))



  ;;; PHASE 3: manipulate vectors to implement collision 

  ;; compute the velocity of the system's center of mass along theta
  set vcm (((mass * v1t) + (mass2 * v2t)) / (mass + mass2) )

  ;; now compute the new velocity for each particle along direction theta.
  ;; velocity perpendicular to theta is unaffected by a collision along theta,
  ;; so the next two lines actually implement the collision itself, in the
  ;; sense that the effects of the collision are exactly the following changes
  ;; in particle velocity.
  set v1t (2 * vcm - v1t)
  set v2t (2 * vcm - v2t)



  ;;; PHASE 4: convert back to normal speed/heading

  ;; now convert my velocity vector into my new speed and heading
  set speed sqrt ((v1t * v1t) + (v1l * v1l))
  ;; if the magnitude of the velocity vector is 0, atan is undefined. but
  ;; speed will be 0, so heading is irrelevant anyway. therefore, in that 
  ;; case we'll just leave it unmodified.
  if v1l != 0 or v1t != 0
    [ set heading (theta - (atan v1l v1t)) ]

  ;; and do the same for other-particle
  set speed-of other-particle sqrt ((v2t * v2t) + (v2l * v2l))
  if v2l != 0 or v2t != 0
    [ set heading-of other-particle (theta - (atan v2l v2t)) ]



  ;; PHASE 5: final updates

  ;; now recolor, since color is based on quantities that may have changed
  recolor
  ask other-particle
    [ recolor ]
end

to recolor  ;; particle procedure
  ifelse speed < (0.5 * 10)
  [
    set color blue
  ]
  [
    ifelse speed > (1.5 * 10)
      [ set color red ]
      [ set color green ]
  ]
end

;;;
;;; drawing procedures
;;;

;; draws the box
to make-box
  ask patches with [ ((abs pxcor = box-edge) and (abs pycor <= box-edge)) or
                     ((abs pycor = box-edge) and (abs pxcor <= box-edge)) ]
    [ set pcolor yellow ]
  ask patches with [pycor = 0 and pxcor < (1 - box-edge)]
  [
    set pcolor yellow - 5  ;; trick the bounce code so particles don't go into the inlet
    set pcolor-of patch-at 0  1 yellow
    set pcolor-of patch-at 0 -1 yellow
  ]
end

;; creates initial particles
to make-particles [number]
  create-custom-particles number
  [
    setup-particle
    set speed random-float 20
    random-position
    recolor
  ]
  calculate-tick-length
end

;; adds particles from the left
to add-particles-side
  if particles-to-add > 0
    [ cct-particles particles-to-add
        [ setup-particle
          setxy (- box-edge) 0
          set heading 90 ;; east
          rt 45 - random-float 90
          recolor
          
        ]
      if announce-add-particles? [ play-note "tubular bells" 59 90 0.3 ]  ;;  announce when particles are added
      set particles-to-add 0
      calculate-tick-length
    ]
end

;; called by student from command center;
;; adds particles in middle
to add-particles-middle [n]
  cct-particles n
    [ setup-particle
      set new? true
      rt random-float 360
      recolor ]
  ;; add the new particles to an agentset, so they
  ;; are accessible to the student from the command
  ;; center, e.g. "ask new-particles [ ... ]"
  set new-particles particles with [new?]
  calculate-tick-length
end

to setup-particle  ;; particle procedure
  set new? false
  set speed 10
  set mass 1.0
  set last-collision nobody
  set wall-hits 0
  set momentum-difference 0
end

;; place particle at random location inside the box.
to random-position ;; particle procedure
  setxy ((1 - box-edge) + random-float ((2 * box-edge) - 2))
        ((1 - box-edge) + random-float ((2 * box-edge) - 2))
  set heading random-float 360
end

;; make turtles appear more like particles
to-report particle [n]
  report turtle n
end

;;; plotting procedures

to setup-plots
  set-current-plot "Number vs. Time"
  set-plot-y-range 0 initial-number + 1
end


to do-plotting

  set-current-plot "Number vs. Time"
  plotxy clock (count particles)
  
  set-current-plot "Pressure vs. Time"
  if length pressure-history > 0
    [
       ifelse avg-plot-pressure?  ;; pressure plot is 'smoothed' by averaging 3 sequential values, only if switch is set this way
         [ plotxy clock (mean last-n 3 pressure-history) ]
         [ plotxy clock pressure ]
     ]
end

to-report last-n [n the-list]
 ifelse n >= length the-list
   [ report the-list ]
   [ report last-n n butfirst the-list ]
end

to make-clocker
  set-default-shape clocker "clocker"
  create-custom-clocker 1
  [
    setxy (box-edge - 5) (box-edge - 5) 
    set color violet + 2
    set size 10 
    set heading 0
  ]
end

to fade-patches
  locals [ trace-patches ]
  set trace-patches patches with [ shade-of? pcolor red or shade-of? pcolor  blue or shade-of? pcolor green ]
  if any? trace-patches
    [ ask trace-patches
      [ set pcolor ( pcolor - 0.05 )
        if (pcolor mod 10 < 1)
          [ set pcolor black ] ] ]
end

to sing-one-particle-speed     ;; procedure for listening to one particle's speed - pitch is a function of speed
   ifelse single-particle-speed?
      [  ifelse (three-speed > 3)
             [ stop-note "oboe" (80 - 120 / ( three-speed )) 
               stop-note "oboe" 50
               start-note "oboe" (80 - 120 / ( speed-of particle 3 )) single-particle-loudness 
             ]
             [ 
               stop-note "oboe" (80 - 120 / ( three-speed )) 
               stop-note "oboe" 50
               start-note "oboe" 50 single-particle-loudness + 20 
             ]
               set three-speed speed-of particle 3            
       ]
       [ 
         ifelse (three-speed > 3)
           [ stop-note "oboe" (80 - 120 / ( three-speed ))]
           [ stop-note "oboe" 50 ]
       ]
end

to sing-real-time ;; real time is drummed at a regular interval
       every real-time-pacer [ play-note "sci-fi" 30 60 0.3 ]  
end

to sing-pressure  ;;  pressure is sung by a recorder with the pitch a function of pressure
  if ( abs ( pressure-now - pressure ) > 0 and pressure-now != 0)       
          [ stop-note "recorder" ( 100 - 2000 / pressure-now )
            stop-note "recorder" ( 30 ) ]
  set pressure-now pressure 
  ifelse (pressure > 30 )
        [ start-note "recorder" ( 100 - 2000 / pressure ) pressure-loudness ]
        [ start-note "recorder" ( 30 ) pressure-loudness + 10 ]
end


; *** NetLogo Model Copyright Notice ***
;
; This activity and associated models and materials were created as part of the project:
; MODELING ACROSS THE CURRICULUM.  The project gratefully acknowledges the support of the 
; National Science Foundation, National Institute of Health and the Department of Education 
; (IERI program) -- grant number REC # 0115699. Additional support was provided through the 
; project: PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN CLASSROOMS 
; -- NSF (REPP program) -- grant number REC #9814682.
;
; Copyright 2002 by Uri Wilensky.  Updated 2004.  All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (2002).  NetLogo GasLab With Sound model.
; http://ccl.northwestern.edu/netlogo/models/GasLabWithSound.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 2002 by Uri Wilensky.  All rights reserved.  See
; http://ccl.northwestern.edu/netlogo/models/GasLabWithSound
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
212
10
490
309
33
33
4.0
1
10
1
1
1

CC-WINDOW
-1
163
209
271
Command Center

MONITOR
0
112
56
161
clock
floor clock
0
1

BUTTON
125
44
211
77
go/stop
go
T
1
T
OBSERVER
T

BUTTON
125
11
211
44
NIL
setup
NIL
1
T
OBSERVER
T

PLOT
491
155
694
309
Number vs. Time
time
number
0.0
20.0
0.0
200.0
true
false

SLIDER
0
78
124
111
number-to-add
number-to-add
0
100
25
1
1
NIL

BUTTON
125
78
211
111
add particles
set particles-to-add\n    particles-to-add +\n      number-to-add
NIL
1
T
OBSERVER
NIL

SLIDER
0
44
124
77
initial-number
initial-number
0
100
25
1
1
NIL

MONITOR
57
112
123
161
Number
count particles
0
1

PLOT
492
10
695
155
Pressure vs. Time
time
pressure
0.0
20.0
0.0
100.0
true
false

MONITOR
124
112
211
161
pressure
pressure
0
1

SWITCH
237
463
350
496
collisions?
collisions?
1
1
-1000

SWITCH
238
428
350
461
wall-hits?
wall-hits?
1
1
-1000

SWITCH
19
462
153
495
model-time?
model-time?
0
1
-1000

SWITCH
19
497
154
530
real-time?
real-time?
1
1
-1000

SWITCH
507
428
696
461
single-particle-speed?
single-particle-speed?
1
1
-1000

SWITCH
508
463
696
496
single-particle-wall-hits?
single-particle-wall-hits?
1
1
-1000

SWITCH
508
497
695
530
single-particle-collisions?
single-particle-collisions?
1
1
-1000

SLIDER
155
497
342
530
real-time-pacer
real-time-pacer
0.5
5
1.325
0.025
1
sec

SWITCH
14
346
204
379
announce-add-particles?
announce-add-particles?
0
1
-1000

TEXTBOX
538
322
654
340
Listen to one particle

SWITCH
238
394
350
427
pressure?
pressure?
1
1
-1000

TEXTBOX
289
322
425
340
Listen to all the particles

TEXTBOX
112
442
202
460
Time in a model

SLIDER
351
463
478
496
collisions-loudness
collisions-loudness
0
100
25
5
1
NIL

SLIDER
352
428
477
461
wall-hits-loudness
wall-hits-loudness
0
100
25
5
1
NIL

SLIDER
352
393
478
426
pressure-loudness
pressure-loudness
0
120
40
5
1
NIL

BUTTON
38
285
103
318
all on
set real-time? true \nset model-time? true \nset pressure? true \nset wall-hits? true \nset collisions? true \nset single-particle-speed? true \nset single-particle-collisions? true\nset single-particle-wall-hits? true \nset announce-add-particles? true
NIL
1
T
OBSERVER
NIL

BUTTON
104
285
170
318
all off
stop-music\nset collisions? false\nset single-particle-speed? false \nset single-particle-collisions? false\nset single-particle-wall-hits? false\nset announce-add-particles? false\nset model-time? false\nset real-time? false\nset pressure? false\nset wall-hits? false
NIL
1
T
OBSERVER
NIL

SLIDER
507
394
696
427
single-particle-loudness
single-particle-loudness
0
80
30
5
1
NIL

MONITOR
465
340
539
389
single speed
speed-of particle 3
1
1

SWITCH
543
349
647
382
single-on?
single-on?
1
1
-1000

SWITCH
298
348
422
381
group-on?
group-on?
1
1
-1000

SWITCH
343
497
508
530
avg-plot-pressure?
avg-plot-pressure?
1
1
-1000

@#$#@#$#@
WHAT IS IT?
-----------
This model simulates pumping up a bicycle tire.  It consists of gas particles that are trapped in a fixed and rigid container.  The model specifically looks at how the number of particles affects the pressure of system.  Air can be added into the container through a valve.  Adding a group of particles into the container increases the number of particles that are moving and colliding with each other and hitting the walls.  

This model includes sounds.  Different events and properties can be listened to. You can listen to some of the sounds or all of them, to the group events or to the individual particle events.
In this activity, you may explore the model to find out how the different events in the model are related, and how they change when you add more and more particles:
* Model time and real time
* Macroscopic variables: pressure, wall hits, collisions
* Microscopic variables: particle speed, particle wall hits, particle collisions

This model is part of the Connected Chemistry High School Curriculum and is part of the second and third activities, in which the concept of pressure is explored.  In this model, students look at the relationship between number of particles and pressure. In addition, one can follow the average number of wall hits in one model clock tick.  A clock hand is used to mark the passage of time in the model.  When the particles hit the walls they change their color temporarily.  These models have been adapted from the model GasLab Pressure Box, which can be found in the Unverified section of the GasLab folder found under Chemistry and Physics.  

These pressure models are part of a suite of models that students use to gain deeper insight on Gas Laws and particle behavior. In all of the Connected Chemistry Curriculum models, the same basic rules are used for expressing what happens when gas particles collide.  Each model has different features in order to show different aspects of the behavior of gases.

This model is part of the Connected Chemistry high school curriculum.  Connected Chemistry is currently under development; for more information please refer to:
http://ccl.northwestern.edu/curriculum/chemistry/


HOW IT WORKS
------------
Particles are modeled as perfectly elastic particles with no energy except their kinetic energy -- which is due to their motion.  Collisions between particles are elastic.  Particles are colored according to speed -- blue for slow, green for medium, and red for high speeds.

The exact way two particles collide is as follows:
1. Two turtles "collide" when they find themselves on the same patch.
2. A random axis is chosen, as if they are two balls that hit each other and this axis is the line connecting their centers.
3. They exchange momentum and energy along that axis, according to the conservation of momentum and energy.  This calculation is done in the center of mass system.
4. Each turtle is assigned its new velocity, energy, and heading.
5. If a turtle finds itself on or very close to a wall of the container, it "bounces" -- that is, reflects its direction and keeps its same speed.

Sounds
The loudness of the announcement on particles entering the box is connected to their number
The pitch on pressure is related to the value of pressure.
Wall hits' pitches are associated with the headings of the hitting particles.
Collisions' pitches are calculated using the sum of the colliding particles' speeds.
The pitch for a single particle's speed is related to its speed.


HOW TO USE IT
-------------
Buttons:
  SETUP - sets up the initial conditions set on the sliders.  Be sure to wait till the Setup button stops before pushing GO/STOP.
  GO/STOP - runs and stops the model.  
  ADD-PARTICLES - when pressed releases particles into the box while the simulation is running.
ALL ON - turns on all the sounds
ALL OFF - turns off all the sounds

Sliders:
  INITIAL-NUMBER - sets the number of gas particles in the box when the simulation starts.
  NUMBER-TO-ADD - the number of gas particles released into the box when the ADD-   PARTICLES button is pressed.
REAL-TIME-PACER - sets the cycle for the real time sound
PRESSURE-LOUDNESS - sets the loudness for pressure's sound
 WALL-HITS-LOUDNESS - sets the loudness for wall hits' sound
 COLLISIONS-LOUDNESS - sets the loudness for collisions' sound
 SINGLE-PARTICLE-LOUDNESS

Switches
The following switches allow adding sounds to the model for the different events
  ANNOUNCE-ADD-PARTICLES?
  MODEL-TIME?
  GROUP-ON?
  PRESSURE?
  WALL-HITS?
  COLLISIONS?
  SINGLE-ON?
  SINGLE-PARTICLE-SPEED?
  SINGLE-PARTICLE-WALL-HITS?
  SINGLE-PARTICLE-COLLISIONS?

Monitors:
  CLOCK - number of clock cycles that GO has run.
  NUMBER - the number of particles in the box.
  PRESSURE - the total pressure in the box.
  WALL HITS PER PARTICLE - the average number of wall hits in one clock tick
  SINGLE-SPEED - speed of one particle (ID 3), which is being tracked by sound and   pen-down

Plots:
  PRESSURE VS TIME - plots the pressure in the box over time.
  NUMBER VS TIME - plots the number of particles in the box over time.  This plot is   
  laid under the WALL HITS PER PARTICLE plot, as in scripted activities the two plots
  can be exposed in different screens.
  WALL HITS PER PARTICLE - the average number of wall hits in one clock tick

Initially, the particles are not moving. Therefore the initial pressure is zero.  When the particles start moving, they repeatedly collide, exchange energy and head off in new directions, and the speeds are dispersed -- some particles get faster, some get slower.  When they hit the wall they change their heading, but not their speed.


THINGS TO NOTICE
----------------
Can you relate what you can see happening to the particles in the box with changes in pressure?  

Why does the pressure change over time, even when the number of particles is the same?  How long does it take for the pressure to stabilize?

What happens to the wall hits per particle when particles are added to the box?

MODEL-TIME: As there are more particles in the model, the model slows down.  Add particles to hear that happen.

Compare MODEL-TIME with REAL-TIME.

Listen to one particle alone (its ID is 3), its speed, collisions and wall hits.  Change the number of particles.  When does the particle change speed more frequently?  When does the particle collide more frequently?  

You can listen to some features that involve the whole group: PRESSURE, WALL HITS and COLLISIONS.  Add particles a number of times and notice what happens to these features.

Listen to PRESSURE: can you hear the sounds go up and down the scale together with the Pressure vs. Time plot or the pressure monitor?  Can you hear the plot with WALL HITS?

Turn on the ANNOUNCE-ADD-PARTICLES?  Do you notice the delay in time between injecting particles and the pressure going up?  Listen to the wall-hits as the particles hit the wall.


In what ways is this model an incorrect idealization of the real world?


THINGS TO TRY
-------------
Try different settings, especially the extremes.   Are the particles behaving in a similar way?  How does this affect the pressure? 

You can pen-down a particle through the command center or by using the turtle menus.  What do you notice about a particle's path when there more and less particles in the box?

Can you make the pressure graph smooth?  Can you do it in more than one way? 

Sometimes, when going up in an elevator, airplane or up a mountain we feel a 'popping' sensation in our ears.  This is associated with changes in pressure.  Can you relate between this model and these changes in pressure?  Are the temperature and volume constant in this situation?


EXTENDING THE MODEL
-------------------
Add a histogram showing all the particles speeds throughout the run of the model.

Add a switch that would continuously add particles to the box?

Make a hole in the box and observe what happens to the pressure.


CREDITS AND REFERENCES
----------------------
To refer to this model in academic publications, please use: Wilensky, U. (2004).  NetLogo GasLabWithSound model.  http://ccl.northwestern.edu/netlogo/models/GasLabWithSound.  Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use: Copyright 2002 by Uri Wilensky.  All rights reserved.  See http://ccl.northwestern.edu/netlogo/models/GasLabWithSound for terms of use.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

circle
false
0
Circle -7566196 true true 35 35 230

clocker
true
0
Polygon -65281 true false 150 30 105 195 135 180 135 270 165 270 165 180 195 195

@#$#@#$#@
NetLogo 2.0.2
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
